@@ -1,7 +1,7 @@
 /* Array bitsets.
 
-   Copyright (C) 2002-2003, 2006, 2009-2013 Free Software Foundation,
-   Inc.
+   Copyright (C) 2002-2003, 2006, 2009-2015, 2018 Free Software
+   Foundation, Inc.
 
    Contributed by Michael Hayes (m.hayes@elec.canterbury.ac.nz).
 
@@ -146,7 +146,7 @@ abitset_list_reverse (bitset src, bitset_bindex *list,
   bitset_bindex rbitno;
   bitset_bindex count;
   bitset_windex windex;
-  unsigned int bitcnt;
+  unsigned bitcnt;
   bitset_bindex bitoff;
   bitset_word *srcp = ABITSET_WORDS (src);
   bitset_bindex n_bits = BITSET_SIZE_ (src);
@@ -302,7 +302,7 @@ abitset_list (bitset src, bitset_bindex *list,
 static inline void
 abitset_unused_clear (bitset dst)
 {
-  unsigned int last_bit;
+  unsigned last_bit;
 
   last_bit = BITSET_SIZE_ (dst) % BITSET_WORD_BITS;
   if (last_bit)
